//
//  SITCommunicationInterface.h
//  SitumSDK
//
//  Created by A Barros on 15/6/16.
//  Copyright © 2016 Situm. All rights reserved.
//

#ifndef SITCommunicationInterface_h
#define SITCommunicationInterface_h

#import "SITIndoorBuilding.h"
#import "SITIndoorLevel.h"

#import "SITOccurrence.h"
#import "SITEvent.h"
#import "SITPOICategory.h"
#import "SITPOI.h"
#import "SITPOIExterior.h"
#import "SITEvent.h"

static NSString* _Nonnull const kSITBuildingInfoKey = @"indoorBuilding";
static NSString* _Nonnull const kSITFloorsInfoKey = @"levels";
static NSString* _Nonnull const kSITIndoorPOIsInfoKey = @"indoor_pois";
static NSString* _Nonnull const kSITOutdoorPOIsInfoKey = @"outdoorPois";
static NSString* _Nonnull const kSITEventsInfoKey = @"events";
static NSString* _Nonnull const kSITGraphInfoKey = @"graph";

@class SITGraph;

typedef void (^SITImageFetchHandler)(NSData * _Nullable imageData);

typedef void (^SITFailureOperationHandler)(NSError * _Nullable error);

/**
 The type of block callback that will get executed when a repository operation has successfully been performed

 @param mapping create a class instead of passing a dictionary??
 @note discuss about the keys of the dictionary
 */
typedef void (^SITSuccessHandler)(NSDictionary * _Nullable mapping);

typedef void (^SITHandler)(id _Nullable result, NSError * _Nullable error);

/*!=
 *  The type of block callback for handling indoor buildings data
 *
 *  @param indoorBuildings An array of objects that encapsulates a collection of SITIndoorBuilding objects based on Authentication
 *  @param error    An error object representing a problem while retrieving information from Internet
 */
typedef void (^SITBuildingFetchHandler)(NSArray * _Nullable indoorBuildings, NSError * _Nullable error) __attribute__((deprecated("Deprecated")));;

typedef void (^SITBuildingInfoFetchHandler)(NSDictionary * _Nullable info, NSError * _Nullable error);

/*!
 *  The type of block callback for handling indoor level data
 *
 *  @param indoorLevels An array encapsulating SITIndoorLevel objects
 *  @param error        An error object representing a problem while retrieving information from Internet
 *
 *  @warning TBC
 */
typedef void (^SITIndoorLevelFetchHandler)(NSArray * _Nullable indoorLevels, NSError * _Nullable error) __attribute__((deprecated("Deprecated")));


/**
 *  The type of block callback for handling indoor level map data
 *
 *  @param indoorLevelMapData An NSData object with information of an indoor level map
 *  @note You can use this object to create UIImageView in order to display them on screen.
 */
typedef void (^SITIndoorLevelMapFetchHandler)(NSData * _Nullable indoorLevelMapData) __attribute__((deprecated("Deprecated")));

/*!
 *  The type of block callback for handling POIs (Point Of Interest) data
 *
 *  @param POIs  An array encapsulating SITPOI objects
 *  @param error An error object representing a problem while retrieving information from Internet
 */
typedef void (^SITPOIFetchHandler)(NSArray * _Nullable POIs, NSError * _Nullable error);

/**
 *  The tyoe of block callback for handling External POIs (Points Of Interest) data
 *
 *  @param exteriorPOIS An array encapsulating SITPOIExterior objects
 *  @param error        An error object representing a problem while retrieving information from Internet
 */
typedef void (^SITExteriorPOIFetchHandler)(NSArray * _Nullable exteriorPOIS, NSError * _Nullable error);

/**
 *  The type of block callback for handling the graph of a SITIndoorBuilding object
 *
 *  @param graphs An array of object of class SITGraph
 *  @param error An error object representing a problem while retrieving information from Internet
 */
typedef void (^SITGraphFetchHandler)(NSArray * _Nullable graphs, NSError * _Nullable error) __attribute__((deprecated("Deprecated")));

/**
 *  The type of block for handling events of a SITIndoorBuilding object
 *
 *  @param events array containing the events inside a building
 *  @param error  An error object representing a problem while retrieving information from Internet
 */
typedef void (^SITEventsFetchHandler)(NSArray<SITEvent*> * _Nullable events, NSError * _Nullable error);

/**
 *  The type of block that will be called after the method - fetchCategoriesWithCompletion: is executed
 *
 *  @param categories An array SITPOICategory class objects
 *  @param error      An error object representing a problem while retrieving information from Internet
 */
typedef void (^SITPOICategoriesFetchHandler)(NSArray * _Nullable categories, NSError * _Nullable error);

/**
 *  The type of block that will be called after an icon of a category has been requested
 *
 *  @param iconData An NSData object containing the data of the icon
 *  @param error    An error object representing a problem while retrieving the data
 */
typedef void (^SITPOICategoryIconFetchCompletion)(NSData * _Nullable iconData, NSError * _Nullable error);

/**
 *  The type of block that will be called after a login operation
 *  @param error    An error object describing what went wrong (if any)
 */
typedef void (^SITLoginHandler)(NSError * _Nullable error);

/**
 *  The type of block that will be called after a logout operation
 *  @param error    An error object describing what went wrong (if any)
 */
typedef void (^SITLogoutHandler)(NSError * _Nullable error);

/**
 *  The type of block callback for handling calibrations updates for a specific floor
 *
 *  @param error An error object representing a problem while performing the operation.
 */
typedef void (^SITCalibrationsPushHandler)(NSError * _Nullable error) __attribute__((deprecated("Deprecated")));

typedef void (^SITOccurrenceCompletion)(SITOccurrence * _Nullable occurrence);

typedef void (^SITErrorCompletion)(NSError * _Nullable error);

typedef void (^SITSuccessRetrievingModelCompletion)(NSData * _Nullable data);

typedef void (^SITFailureCompletion)(NSError * _Nullable error);

#endif /* SITCommunicationInterface_h */
