//
//  SITCoordinateConverter.h
//  SitumSDK
//
//  Created by A Barros on 2/3/17.
//  Copyright © 2017 Situm. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "SITAngle.h"
#import "SITLocation.h"
#import "SITPoint.h"
#import "SITDimensions.h"
#import "SITBounds.h"

/**
 Utility class to convert between cartesian coordinates and geographical coordinates.
 */
@interface SITCoordinateConverter : NSObject

#pragma mark - Initializers

/**
 Initializer
 
 @param dimensions Dimensions of the building
 @param center Center of the building
 @param rotation Rotation of the building
 
 @return `SITCoordinateConverter`
 */
- (instancetype  _Nonnull)initWithDimensions:(SITDimensions * _Nonnull)dimensions
                                      center:(CLLocationCoordinate2D)center
                                    rotation:(SITAngle * _Nonnull)rotation;

#pragma mark - Methods

/**
 Dimensions of cartesian area, in meters
 @return `SITDimensions`
 */
- (SITDimensions* _Nonnull) getDimensions;

/**
 Convert geographic coordinate to cartesian coordinate
 @return `SITCartesianCoordinate`
 */
- (SITCartesianCoordinate * _Nonnull)toCartesianCoordinate:(CLLocationCoordinate2D)coordinate;

/**
 Convert cartesian coordinate to geographic coordinate
 @return `CLLocationCoordinate2D`
 */
- (CLLocationCoordinate2D)toLocationCoordinate:(SITCartesianCoordinate * _Nonnull)cartesianCoordinate;

/**
 Converts an angle from building coordinate system to Earth coordinate system.
 @return `SITAngle`
 */
- (SITAngle * _Nonnull)toAngle:(SITAngle * _Nonnull)cartesianAngle;

/**
 Converts an angle from Earth coordinate system to building coordinate system.
 @return `SITAngle`
 */
- (SITAngle * _Nonnull)toCartesianAngle:(SITAngle * _Nonnull)angle;

/**
 Building bounds
 */
- (SITBounds)bounds;

/**
 Rotated bounds
 */
- (SITBounds)rotatedBounds;

/// :nodoc:
- (SITAngle* _Nonnull) convertBearingToCartesianAngle: (SITAngle* _Nonnull) bearing;

@end
